
class Keys:
    # Config component
    INT = 'int'
    VCO = 'vco'
    LINE = 'line'
    EXT = 'ext'

    RISE = 'rise'
    FALL = 'fall'
    SINE = 'sine'

    SHAFT = 'shaft'
    INNER = 'inner'
    OUTER = 'outer'

    # Operate component
    OFF = 'off'
    ON = 'on'

    SOURCE = 'source'
    SUM = 'sum'
    DIFF = 'diff'
    CTRL = 'ctrl'

    INTERNAL = 'internal'
    PHASE = 'phase'
    MULTIN = 'multin'
    DIVM = 'divm'
    VCOFS = 'vcofs'

    # Status component
    ESB = 'ESB'
    MSS = 'MSS'
    CHSB = 'CHSB'

    OPC = 'Operation complete'
    INP = 'Input Buffer Overrun'
    QYE = 'Query Error'
    DDE = 'Device Dependent Error'
    EXE = 'Execution Error'
    CME = 'Command Error'
    URQ = 'User Request'
    PON = 'Power On'

    MON = 'MON'
    EL = 'EL'
    FL = 'FL'
    PL = 'PL'
    CMAX = 'CMAX'
    TMAX = 'TMAX'

    ChopperHeadMemoryFail = 'chopper head memory fail'
    ChopperHeadDisconnect = 'chopper head disconnect'
